@@ -9,4 +9,10 @@
 extern crate libc;
 extern crate errno;
 
-pub mod libproc;
\ No newline at end of file
+pub mod libproc;
+
+#[cfg(target_os = "macos")]
+#[allow(warnings, missing_docs)]
+mod osx_libproc_bindings {
+include!(concat!(env!("OUT_DIR"), "/osx_libproc_bindings.rs"));
+}
